BeginPackage["HEPMath`Color`", {"HEPMath`Tensors`"}]

ColorFundamental::usage = "ColorFundamental is the index type of color indices
in the fundamental representation."

ColorAdjoint::usage = "ColorAdjoint is the index type of color indices
in the adjoint representation."

ColorT::usage = "ColorT represents the generators of the SU(3) color algebra.
Its tensor signature is {ColorAdjoint, ColorFundamental, ColorFundamental}"

ColorTr::usage = "ColorTr represents the trace over color indices."

ColorF::usage = "ColorF represents the structure constants of the SU(3) color
algebra. Its tensor signature is {ColorAdjoint, ColorAdjoint, ColorAdjoint}."

ColorDelta::usage = "ColorDelta represents the identity matrix in the space of
fundamental color indices. Its tensor signature is {ColorFundamental,
ColorFundamental}"

GluonDelta::usage = "GluonDelta represents the identity matrix in the
space of adjoint color indices. Its tensor signature is {ColorAdjoint,
ColorAdjoint}"

ColorReduce::usage = "ColorReduce[expr] reduces an expression expr with no free
color indices to an expression with no color indices."

ColorReduce::nosup = "ColorReduce only works on expressions without suppressed
ColorFundamental and ColorAdjoint indices."


Begin["`Private`"]

DeclareIndexType[ColorFundamental, 3, ColorDelta]

DeclareIndexType[ColorAdjoint, 8, GluonDelta]

DeclareHEPTensor[ColorT, {ColorAdjoint, ColorFundamental, ColorFundamental}]

DeclareRealHEPTensor[ColorF, {ColorAdjoint, ColorAdjoint, ColorAdjoint}]

DeclareLinear[ColorTr]

CC[ColorTr[HoldPattern[Dot[aa__ColorT]]]] := ColorTr[Dot@@Reverse[{aa}]]

CC[ColorTr[a_]] := ColorTr[CC[a]]

UserExplicitIndices[e:ColorTr[t_], patt_] := Block[{i=Unique[]},
  ExplicitIndices[t[i, i], patt]] /; !FreeQ[e, patt]

CC[ColorT][a_, i_, j_] := ColorT[a, j, i]


(* ColorReduce[ex_] := Block[{res}, *)
(*   res = HEPReplaceAll[ex, ColorF[a_, b_, c_] -> *)
(*     -2 I (ColorT[a,i1,i2] ColorT[b,i2,i3] - ColorT[b,i1,i2] ColorT[a,i2,i3]) * *)
(*     ColorT[c,i3,i1]]; *)

(*   res = FixedPoint[Expand[# //. ColorT[a_, i_, j_] ColorT[a_, k_, l_] -> *)
(*       ColorDelta[i,l] ColorDelta[k,j]/2 - ColorDelta[i,j] ColorDelta[k,l]/6, *)
(*     ColorT|ColorDelta|ColorAdjDelta]&, res]; *)

(*   HEPContract[res] *)
(* ] *)

ColorReduce[ex_] := (Message[ColorReduce::nosup]; ex) /;
  !FreeQ[HEPTensorSignature[ex], ColorFundamental|ColorAdjoint]

ColorReduce[ex_] := Block[{col, coeff},
  {col, coeff} = HEPCollectListed[ex,
    ColorT|ColorF|ColorDelta|GluonDelta|ColorTr];
  col = colorReduce /@ col;
  col.coeff
]

colorReduce[ex_] := Block[{res=ex, colT},
  (* translate to internal representation *)
  res = HEPContract[res, ColorDelta|GluonDelta];
  res = res /. ColorDelta[i_,j_] -> colT[][i,j];
  res = res /. ColorT[a_] -> colT[a];
  res = res //. colT[aa__].colT[bb__] -> colT[aa,bb];
  res = res /. ColorTr[colT[aa___]] -> colT[aa][];
  res = res /. ColorT[a_, i_, j_] -> colT[a][i,j];
  res = res //. colT[aa___][i_, j_] colT[bb___][j_, k_] -> colT[aa,bb][i,k];
  res = res /. colT[aa___][i_, i_] -> colT[aa][];
  res = res /. ColorF[a_, b_, c_] -> -2 I colT[a,b,c][] + 2 I colT[b,a,c][];

  (* apply T^a_ij T^b_kl = d_il d_kj/2 - d_ij d_kl/6 identity *)
  res = Expand[res, colT];
  res = FixedPoint[Expand[# /. {
    colT[c1___, a_, c2___, a_, c3___][ii___] ->
      colT[c1, c3][ii] colT[c2][]/2 - colT[c1, c2, c3][ii]/6,
    colT[c1___, a_, c2___][i_,j_] colT[c3___, a_, c4___][k_,l_] ->
      colT[c1, c4][i,l] colT[c3, c2][k,j]/2 -
      colT[c1, c2][i,j] colT[c3, c4][k,l]/6,
    colT[c1___, a_, c2___][i_,j_] colT[c3___, a_, c4___][] ->
      colT[c1, c4, c3, c2][i,j]/2 - colT[c1, c2][i,j] colT[c3, c4][]/6,
    colT[c1___, a_, c2___][] colT[c3___, a_, c4___][] ->
      colT[c1, c4, c3, c2][]/2 - colT[c1, c2][] colT[c3, c4][]/6,
    colT[a_, c___][]^2 -> colT[c,c][]/2 - colT[c][]^2/6
    }, colT]&, res];
  res = res /. {colT[][] -> 3, colT[][i_,j_] -> ColorDelta[i,j]};

  (* order color traces canonically *)
  res = res /. colT[aa__][] :> Block[{amin=First[Sort[{aa}]]},
    (colT@@Replace[{aa}, {b1___, amin, b2___} -> {amin, b2, b1}])[]
  ];

  (* switch back to external representation *)
  res /. {
    colT[aa__][] :> ColorTr[Dot @@ (ColorT /@ {aa})],
    colT[aa__][i_,j_] :> (Dot @@ (ColorT /@ {aa}))[i,j]
  } /. ColorT[a_][i_, j_] -> ColorT[a,i,j]
]

End[]

EndPackage[]
