#include "WolframLibrary.h"
#include "clooptools.h"

static const mreal epsilon = 1.e-14;

static int initialize = 1;

EXTERN_C DLLEXPORT mint WolframLibrary_getVersion()
{
    return WolframLibraryVersion;
}

EXTERN_C DLLEXPORT int WolframLibrary_initialize(WolframLibraryData libData)
{
    if(initialize == 1) {
	ltini();
	setdelta(0.);
	setmudim(1.);
	setlambda(0.);

	initialize = 0;
    }

    return 0;
}

EXTERN_C DLLEXPORT void WolframLibrary_uninitialize(WolframLibraryData libData)
{
    if(initialize == 0) {
	ltexi();

	initialize = 1;
    }

    return;
}

EXTERN_C DLLEXPORT int GetAIntegrals(WolframLibraryData libData,
				     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal eps;
    mreal msq;
    MTensor res;
    mint dims[1];
    int err = LIBRARY_NO_ERROR;

    if(Argc != 2) {
    	libData->Message("One-point integrals require two arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    msq = MArgument_getReal(Args[1]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(msq) < eps) msq = 0.;

    dims[0] = Naa;
    err = libData->MTensor_new(MType_Complex, 1, dims, &res);
    if(err) return err;
    
    setmudim(musq);
    setlambda(0.);
    Aput(reinterpret_cast<ComplexType*>(libData->MTensor_getComplexData(res)),
	 msq);
    clearcache();

    MArgument_setMTensor(Res, res);
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetA0(WolframLibraryData libData,
			     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal lsq;
    mreal eps;
    mreal msq;
    ComplexType res;
    int err = LIBRARY_NO_ERROR;

    if(Argc != 3) {
    	libData->Message("Scalar one-point integrals require three arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    lsq = -MArgument_getInteger(Args[1]);
    msq = MArgument_getReal(Args[2]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(msq) < eps) msq = 0.;

    setmudim(musq);
    setlambda(lsq);
    res = A0(msq);
    clearcache();

    MArgument_setComplex(Res, reinterpret_cast<mcomplex&>(res));
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetBIntegrals(WolframLibraryData libData,
				     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal eps;
    mreal psq, m0sq, m1sq;
    MTensor res;
    mint dims[1];
    int err = LIBRARY_NO_ERROR;

    if(Argc != 4) {
    	libData->Message("Two-point integrals require four arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    psq = MArgument_getReal(Args[1]);
    m0sq = MArgument_getReal(Args[2]);
    m1sq = MArgument_getReal(Args[3]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(psq) < eps) psq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;

    dims[0] = Nbb;
    err = libData->MTensor_new(MType_Complex, 1, dims, &res);
    if(err) return err;
    
    setmudim(musq);
    setlambda(0.);
    Bput(reinterpret_cast<ComplexType*>(libData->MTensor_getComplexData(res)),
	 psq, m0sq, m1sq);
    clearcache();

    MArgument_setMTensor(Res, res);
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetB0(WolframLibraryData libData,
			     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal lsq;
    mreal eps;
    mreal psq, m0sq, m1sq;
    ComplexType res;
    int err = LIBRARY_NO_ERROR;

    if(Argc != 5) {
    	libData->Message("Scalar two-point integrals require five arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    lsq = -MArgument_getInteger(Args[1]);
    psq = MArgument_getReal(Args[2]);
    m0sq = MArgument_getReal(Args[3]);
    m1sq = MArgument_getReal(Args[4]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(psq) < eps) psq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;

    setmudim(musq);
    setlambda(lsq);
    res = B0(psq, m0sq, m1sq);
    clearcache();

    MArgument_setComplex(Res, reinterpret_cast<mcomplex&>(res));
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetCIntegrals(WolframLibraryData libData,
				     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal eps;
    mreal p1sq, p2sq, p12sq, m0sq, m1sq, m2sq;
    MTensor res;
    mint dims[1];
    int err = LIBRARY_NO_ERROR;

    if(Argc != 7) {
    	libData->Message("Three-point integrals require seven arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    p1sq  = MArgument_getReal(Args[1]);
    p2sq  = MArgument_getReal(Args[2]);
    p12sq = MArgument_getReal(Args[3]);
    m0sq  = MArgument_getReal(Args[4]);
    m1sq  = MArgument_getReal(Args[5]);
    m2sq  = MArgument_getReal(Args[6]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(p1sq) < eps) p1sq = 0.;
    if(std::abs(p2sq) < eps) p2sq = 0.;
    if(std::abs(p12sq) < eps) p12sq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;
    if(std::abs(m2sq) < eps) m2sq = 0.;

    dims[0] = Ncc;
    err = libData->MTensor_new(MType_Complex, 1, dims, &res);
    if(err) return err;
    
    setmudim(musq);
    setlambda(0.);
    Cput(reinterpret_cast<ComplexType*>(libData->MTensor_getComplexData(res)),
	 p1sq, p2sq, p12sq, m0sq, m1sq, m2sq);
    clearcache();

    MArgument_setMTensor(Res, res);
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetC0(WolframLibraryData libData,
			     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal lsq;
    mreal eps;
    mreal p1sq, p2sq, p12sq, m0sq, m1sq, m2sq;
    ComplexType res;
    int err = LIBRARY_NO_ERROR;

    if(Argc != 8) {
    	libData->Message("Scalar three-point integrals require eight arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    lsq = -MArgument_getInteger(Args[1]);
    p1sq  = MArgument_getReal(Args[2]);
    p2sq  = MArgument_getReal(Args[3]);
    p12sq = MArgument_getReal(Args[4]);
    m0sq  = MArgument_getReal(Args[5]);
    m1sq  = MArgument_getReal(Args[6]);
    m2sq  = MArgument_getReal(Args[7]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(p1sq) < eps) p1sq = 0.;
    if(std::abs(p2sq) < eps) p2sq = 0.;
    if(std::abs(p12sq) < eps) p12sq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;
    if(std::abs(m2sq) < eps) m2sq = 0.;

    setmudim(musq);
    setlambda(lsq);
    res = C0(p1sq, p2sq, p12sq, m0sq, m1sq, m2sq);
    clearcache();

    MArgument_setComplex(Res, reinterpret_cast<mcomplex&>(res));
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetDIntegrals(WolframLibraryData libData,
				     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal eps;
    mreal p1sq, p2sq, p3sq, p123sq, p12sq, p23sq, m0sq, m1sq, m2sq, m3sq;
    MTensor res;
    mint dims[1];
    int err = LIBRARY_NO_ERROR;

    if(Argc != 11) {
    	libData->Message("Four-point integrals require 11 arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    p1sq   = MArgument_getReal(Args[1]);
    p2sq   = MArgument_getReal(Args[2]);
    p3sq   = MArgument_getReal(Args[3]);
    p123sq = MArgument_getReal(Args[4]);
    p12sq  = MArgument_getReal(Args[5]);
    p23sq  = MArgument_getReal(Args[6]);
    m0sq   = MArgument_getReal(Args[7]);
    m1sq   = MArgument_getReal(Args[8]);
    m2sq   = MArgument_getReal(Args[9]);
    m3sq   = MArgument_getReal(Args[10]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(p1sq) < eps) p1sq = 0.;
    if(std::abs(p2sq) < eps) p2sq = 0.;
    if(std::abs(p3sq) < eps) p3sq = 0.;
    if(std::abs(p123sq) < eps) p123sq = 0.;
    if(std::abs(p12sq) < eps) p12sq = 0.;
    if(std::abs(p23sq) < eps) p23sq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;
    if(std::abs(m2sq) < eps) m2sq = 0.;
    if(std::abs(m3sq) < eps) m3sq = 0.;

    dims[0] = Ndd;
    err = libData->MTensor_new(MType_Complex, 1, dims, &res);
    if(err) return err;
    
    setmudim(musq);
    setlambda(0.);
    Dput(reinterpret_cast<ComplexType*>(libData->MTensor_getComplexData(res)),
	 p1sq, p2sq, p3sq, p123sq, p12sq, p23sq, m0sq, m1sq, m2sq, m3sq);
    clearcache();

    MArgument_setMTensor(Res, res);
    return LIBRARY_NO_ERROR;
}

EXTERN_C DLLEXPORT int GetD0(WolframLibraryData libData,
			     mint Argc, MArgument *Args, MArgument Res)
{
    mreal mu, musq;
    mreal lsq;
    mreal eps;
    mreal p1sq, p2sq, p3sq, p123sq, p12sq, p23sq, m0sq, m1sq, m2sq, m3sq;
    ComplexType res;
    int err = LIBRARY_NO_ERROR;

    if(Argc != 12) {
    	libData->Message("Scalar four-point integrals require 12 arguments.");
    	return LIBRARY_FUNCTION_ERROR;
    }
    mu = MArgument_getReal(Args[0]);
    lsq = -MArgument_getInteger(Args[1]);
    p1sq   = MArgument_getReal(Args[0]);
    p2sq   = MArgument_getReal(Args[1]);
    p3sq   = MArgument_getReal(Args[2]);
    p123sq = MArgument_getReal(Args[3]);
    p12sq  = MArgument_getReal(Args[4]);
    p23sq  = MArgument_getReal(Args[5]);
    m0sq   = MArgument_getReal(Args[6]);
    m1sq   = MArgument_getReal(Args[7]);
    m2sq   = MArgument_getReal(Args[8]);
    m3sq   = MArgument_getReal(Args[9]);

    musq = mu*mu;
    eps = musq*epsilon;
    if(std::abs(p1sq) < eps) p1sq = 0.;
    if(std::abs(p2sq) < eps) p2sq = 0.;
    if(std::abs(p3sq) < eps) p3sq = 0.;
    if(std::abs(p123sq) < eps) p123sq = 0.;
    if(std::abs(p12sq) < eps) p12sq = 0.;
    if(std::abs(p23sq) < eps) p23sq = 0.;
    if(std::abs(m0sq) < eps) m0sq = 0.;
    if(std::abs(m1sq) < eps) m1sq = 0.;
    if(std::abs(m2sq) < eps) m2sq = 0.;
    if(std::abs(m3sq) < eps) m3sq = 0.;

    setmudim(musq);
    setlambda(lsq);
    res = D0(p1sq, p2sq, p3sq, p123sq, p12sq, p23sq, m0sq, m1sq, m2sq, m3sq);
    clearcache();

    MArgument_setComplex(Res, reinterpret_cast<mcomplex&>(res));
    return LIBRARY_NO_ERROR;
}

